@@ -1,6 +1,7 @@
 use crate::lang::c::{CType, Constant, Function};
 use crate::patterns::LibraryPattern;
-use crate::util::ctypes_from_functions;
+use crate::util::{ctypes_from_functions, extract_namespaces_from_types};
+use std::collections::HashSet;
 
 /// Represents all FFI-relevant items, produced via [`inventory_function`](crate::inventory_function), ingested by backends.
 #[derive(Clone, Debug, PartialOrd, PartialEq)]
@@ -9,6 +10,7 @@ pub struct Library {
     ctypes: Vec<CType>,
     constants: Vec<Constant>,
     patterns: Vec<LibraryPattern>,
+    namespaces: Vec<String>,
 }
 
 impl Library {
@@ -17,6 +19,15 @@ impl Library {
     /// Type information will be automatically derived from the used fields and parameters.
     pub fn new(functions: Vec<Function>, constants: Vec<Constant>, patterns: Vec<LibraryPattern>) -> Self {
         let mut ctypes = ctypes_from_functions(&functions);
+        let mut namespaces = HashSet::new();
+
+        // Extract namespace information
+        extract_namespaces_from_types(&ctypes, &mut namespaces);
+        namespaces.extend(functions.iter().map(|x| x.meta().namespace().to_string()));
+        namespaces.extend(constants.iter().map(|x| x.meta().namespace().to_string()));
+
+        let mut namespaces = namespaces.iter().cloned().collect::<Vec<String>>();
+        namespaces.sort();
 
         // Dont sort functions
         // functions.sort();
@@ -29,6 +40,7 @@ impl Library {
             ctypes,
             constants,
             patterns,
+            namespaces,
         }
     }
 
@@ -48,6 +60,11 @@ impl Library {
         &self.constants
     }
 
+    /// Return all known namespaces.
+    pub fn namespaces(&self) -> &[String] {
+        &self.namespaces
+    }
+
     /// Return all registered [`LibraryPattern`]. In contrast, [`TypePattern`](crate::patterns::TypePattern)
     /// will be found inside the types returned via [`ctypes()`](Self::ctypes).
     pub fn patterns(&self) -> &[LibraryPattern] {
