@@ -17,25 +17,26 @@
 //!
 //! If you ...
 //!
-//! - **have** an `extern "C"` API in Rust, and **need** C#, Python, C, ... bindings to your library,
+//! - **have** an `extern "C"` API in Rust,
+//! - **need** C#, Python, C, ... bindings to your library,
 //! - prefer having **fine-grained-control** over your API and interop generation,
 //! - would like to use **quality-of-life [patterns](crate::patterns)** on **both sides** (e.g., [options](crate::patterns::option), [slices](crate::patterns::slice), '[classes](crate::patterns::class)') where feasible,
-//! - might need to swiftly **support a new language** (3-4 hours to get basic interop generation for new language),
-//! - think your FFI [**single source of truth**](https://en.wikipedia.org/wiki/Single_source_of_truth) should be living Rust code (incl. documentation),
+//! - might need to swiftly **support a new language**,
+//! - think your FFI [**single source of truth**](https://en.wikipedia.org/wiki/Single_source_of_truth) should be living Rust code,
 //!
 //! ... then Interoptopus might be for you.
 //!
 //!
-//! ## Known limitations
+//! ## Known Limitations
 //!
 //! - not yet used in production
-//! - somewhat verbose if you don't own most of your types (still possible, just more work)
+//! - a bit verbose if you don't own your types (still possible, just more work)
 //! - if you target only a single language and don't care about your FFI layer other solutions might be better
 //!
 //!
-//! ## Example & Backends
+//! ## Rust Code You Write
 //!
-//! Assume you have written this Rust FFI code:
+//! This is code you would write:
 //!
 //! ```rust
 //! use interoptopus::{ffi_function, ffi_type};
@@ -57,7 +58,9 @@
 //! interoptopus::inventory_function!(ffi_inventory, [], [my_function], []);
 //! ```
 //!
-//! You can now use one of these backends to generate interop code:
+//! ## Generated Code
+//!
+//! Once you've written the code above you use one of these backends to generate interop code:
 //!
 //! | Language | Crate | Sample Output |
 //! | --- | --- | --- |
@@ -68,7 +71,15 @@
 //!
 //! ## Features
 //!
-//! See the [reference project](https://github.com/ralfbiedert/interoptopus/tree/master/interoptopus_reference_project/src) for a list of all supported features.
+//! See the [**reference project**](https://github.com/ralfbiedert/interoptopus/tree/master/interoptopus_reference_project/src) lists all supported constructs including:
+//! - [functions](https://github.com/ralfbiedert/interoptopus/blob/master/interoptopus_reference_project/src/functions.rs) (`extern "C"` functions and delegates)
+//! - [types](https://github.com/ralfbiedert/interoptopus/blob/master/interoptopus_reference_project/src/types.rs) (primitives, composite, enums (numeric only), opaques, references, pointers, ...)
+//! - [constants](https://github.com/ralfbiedert/interoptopus/blob/master/interoptopus_reference_project/src/constants.rs) (primitive constants; results of const evaluation)
+//! - [patterns](https://github.com/ralfbiedert/interoptopus/tree/master/interoptopus_reference_project/src/patterns) (ASCII pointers, options, slices, classes, ...)
+//!
+//! As a rule of thumb we recommend to be slightly conservative with your signatures and always "think C", since other languages don't track lifetimes
+//! well and it's is easy to accidentally pass an outlived pointer or doubly alias a `&mut X` on reentrant functions.
+//!
 //!
 //! ## Current Status
 //!
@@ -211,23 +222,33 @@ macro_rules! inventory_function {
             let mut constants: Vec<$crate::lang::c::Constant> = Vec::new();
             $(
                 {
-                    use $const as x;
-                    constants.push(x::constant_info());
+                    use $const as user_constant;
+                    constants.push(user_constant::constant_info());
                 }
             )*
 
             let mut functions: Vec<$crate::lang::c::Function> = Vec::new();
             $(
                 {
-                    use $function as x;
-                    functions.push(x::function_info());
+                    use $function as user_function;
+                    functions.push(user_function::function_info());
                 }
             )*
 
             let mut patterns: Vec<$crate::patterns::LibraryPattern> = Vec::new();
             $(
                 {
-                    patterns.push($pattern().into());
+                    let pattern: $crate::patterns::LibraryPattern = $pattern().into();
+
+                    match &pattern {
+                        $crate::patterns::LibraryPattern::Class(class) => {
+                            functions.push(class.constructor().clone());
+                            functions.push(class.destructor().clone());
+                            functions.extend(class.methods().iter().cloned());
+                        }
+                    }
+
+                    patterns.push(pattern);
                 }
             )*
 
