@@ -29,8 +29,45 @@
 //! - the value for `mmR` will be exactly the value returned by a previous constructor invocation,
 
 use crate::lang::c::{CType, Function, OpaqueType};
+use crate::patterns::success_enum::Success;
 use crate::patterns::TypePattern;
 
+macro_rules! impl_failure_default {
+    ($t:ty, $x:expr) => {
+        impl FailureDefault for $t {
+            fn failure_default() -> Self {
+                $x
+            }
+        }
+    };
+}
+
+/// For types that can be returned by a generated class what to return when things went wrong?
+pub trait FailureDefault {
+    fn failure_default() -> Self;
+}
+
+impl<T> FailureDefault for T
+where
+    T: Success,
+{
+    fn failure_default() -> Self {
+        T::NULL
+    }
+}
+
+impl_failure_default!(u8, 0);
+impl_failure_default!(u16, 0);
+impl_failure_default!(u32, 0);
+impl_failure_default!(u64, 0);
+impl_failure_default!(i8, 0);
+impl_failure_default!(i16, 0);
+impl_failure_default!(i32, 0);
+impl_failure_default!(i64, 0);
+impl_failure_default!(f32, f32::NAN);
+impl_failure_default!(f64, f64::NAN);
+impl_failure_default!((), ());
+
 /// Combines a receiver, constructor, destructor and multiple methods in one entity.
 #[derive(Clone, Debug, Ord, PartialOrd, Eq, PartialEq, Hash)]
 pub struct Class {
@@ -146,7 +183,7 @@ impl Class {
 /// );
 /// ```
 #[macro_export]
-macro_rules! pattern_class {
+macro_rules! pattern_class_manual {
     (
         $pattern_name:ident,
         $constructor:path,
@@ -155,7 +192,7 @@ macro_rules! pattern_class {
             $method:path
         ),*]
     ) => {
-        fn $pattern_name() -> interoptopus::patterns::class::Class {
+        pub(crate) fn $pattern_name() -> interoptopus::patterns::class::Class {
             use interoptopus::lang::rust::CTypeInfo;
             use interoptopus::lang::rust::FunctionInfo;
 
@@ -188,6 +225,79 @@ macro_rules! pattern_class {
     };
 }
 
+/// Defines a [`Class`] pattern _and_ generate FFI wrapper code.
+#[macro_export]
+macro_rules! pattern_class_generated {
+    (
+        $export_function:ident,
+        $opaque:ty,
+        $ctor:ident($($ctor_param:ident: $ctor_param_type:ty),*) -> $ctor_error:ty: $method_ctor:ident,
+        $dtor:ident() -> $dtor_error:ty,
+        [
+            $(
+                $method_as_fn:ident($self_ty:ty $(, $param:ident: $param_type:ty)*) -> $t:ty: $method:ident
+            ),*
+        ],
+        [
+            $(
+                $manual_method:ident
+            ),*
+        ]
+    ) => {
+        #[interoptopus::ffi_function]
+        #[no_mangle]
+        pub extern "C" fn $ctor(context_ptr: Option<&mut *mut $opaque>, $( $ctor_param: $ctor_param_type),*) -> $ctor_error {
+            if let Some(context) = context_ptr {
+                let boxed = Box::new(<$opaque>::$method_ctor($($ctor_param),*));
+                let raw = Box::into_raw(boxed);
+
+                *context = raw;
+
+                <$ctor_error as interoptopus::patterns::success_enum::Success>::SUCCESS
+            } else {
+                <$ctor_error as interoptopus::patterns::success_enum::Success>::NULL
+            }
+        }
+
+        #[interoptopus::ffi_function]
+        #[no_mangle]
+        pub extern "C" fn $dtor(context_ptr: Option<&mut *mut $opaque>) -> $dtor_error {
+            if let Some(context) = context_ptr {
+
+                {
+                    unsafe { Box::from_raw(*context) };
+                }
+
+                *context = std::ptr::null_mut();
+
+                <$dtor_error as interoptopus::patterns::success_enum::Success>::SUCCESS
+            } else {
+                <$dtor_error as interoptopus::patterns::success_enum::Success>::NULL
+            }
+        }
+
+        $(
+            #[interoptopus::ffi_function]
+            #[no_mangle]
+            pub extern "C" fn $method_as_fn(context_ptr: Option<$self_ty>, $( $param: $param_type),* ) -> $t {
+                if let Some(context) = context_ptr {
+                    let rval = <$opaque>::$method(context, $($param),*);
+                    rval.into()
+                } else {
+                    < $t as interoptopus::patterns::class::FailureDefault > :: failure_default()
+                }
+            }
+        )*
+
+
+        // Generate export function
+        interoptopus::pattern_class_manual!($export_function, $ctor, $dtor, [
+            $($method_as_fn),*
+            $(,$manual_method)*
+        ]);
+    };
+}
+
 /// Walks the type until it finds the first "obvious" Opaque.
 ///
 /// An Opaque is obvious if it is at a singular position (e.g., `*const Opaque`),
