@@ -14,7 +14,8 @@ pub struct FFISlice<'a, T> {
 }
 
 impl<'a, T> FFISlice<'a, T> {
-    pub fn from_slice(slice: &[T]) -> Self {
+    /// Create new Self from a normal slice.
+    pub fn from_slice(slice: &'a [T]) -> Self {
         FFISlice {
             data: slice.as_ptr(),
             len: slice.len() as u64,
@@ -22,6 +23,7 @@ impl<'a, T> FFISlice<'a, T> {
         }
     }
 
+    /// Tries to return a slice if the pointer was not null.
     pub fn as_slice(&'a self) -> Option<&'a [T]> {
         if self.data.is_null() {
             None
@@ -33,6 +35,23 @@ impl<'a, T> FFISlice<'a, T> {
     }
 }
 
+impl<'a, T> From<&'a [T]> for FFISlice<'a, T> {
+    fn from(slice: &'a [T]) -> Self {
+        Self::from_slice(slice)
+    }
+}
+
+impl<'a, T> FFISlice<'a, T>
+where
+    T: 'static,
+{
+    /// Creates a new empty slice.
+    pub fn empty() -> Self {
+        let x: &'static [T] = &[];
+        Self::from_slice(x)
+    }
+}
+
 impl<'a, T> Deref for FFISlice<'a, T> {
     type Target = [T];
 
